﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Web.Script.Serialization;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using MongoDB.Driver;

namespace WPFApplication
{
   
    public partial class Window1 : Window
    {
        private DashboardService _dashboardService; 

        public Window1()
        {
            _dashboardService = new DashboardService();

            Closed += (Window1_Closed);

            InitializeComponent();
        }

        void Window1_Closed(object sender, EventArgs e)
        {
            _dashboardService.Disconnect();
        }

        public string DatabaseName
        {
            get
            {
                return txtDatabaseName.Text.Trim(); 
            }
        }

        public string SelectedCollectionName
        {
            get
            {
                return (ddlCollections.SelectedValue as String).Split('.')[1]; 
            }
        }

         private void txtDatabaseName_LostFocus(object sender, RoutedEventArgs e)
        {
             ddlCollections.ItemsSource = _dashboardService.GetCollectionNames(DatabaseName); 
        }
       
        private void btn_View_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                BindCollection();
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.Message); 
            }
        }

        private void BindCollection()
        {
            var objs = _dashboardService.GetAllDocumentsAsObservableCollection(DatabaseName, SelectedCollectionName); 
            
            var gridView = new GridView();

            if(objs.Count == 0) throw new ArgumentException("No items exists!");

            var dic = objs[0]; // get the first item
            
            var enumerator = dic.GetEnumerator();

            DataTemplate template = null; 

            while (enumerator.MoveNext())
            {
                var current = enumerator.Current;
                    
                if(current.Value is String)
                {
                    var factory = new FrameworkElementFactory(typeof (TextBlock)); 
                    factory.SetBinding(TextBlock.TextProperty,new Binding("[" + current.Key + "]")); 
                    template = new DataTemplate() { VisualTree = factory}; 
                }

                if(current.Value is ArrayList)
                {
                    template = (DataTemplate) FindResource("GridViewTreeViewDataTemplate");
                    template.LoadContent();

                    var treeView = FindVisualChild<TreeView>(this);
                    treeView.Background = Brushes.Green; 


                    //var factory = new FrameworkElementFactory(typeof (TreeView));

                    //factory.SetBinding(TreeView.ItemsSourceProperty, new Binding("[" + current.Key + "]"));
                    //factory.SetBinding(TreeView.DisplayMemberPathProperty, new Binding("Text")); 

                    //template = new DataTemplate() {VisualTree = factory}; 
                }

                var column = new GridViewColumn
                                 {
                                     Header = current.Key,
                                     CellTemplate = template
                                 };

                                gridView.Columns.Add(column);
            }


            var deleteColumn = new GridViewColumn()
                                   {
                                       CellTemplate = (DataTemplate) FindResource("DeleteButtonDataTemplate")
                                   }; 

            gridView.Columns.Add(deleteColumn);

            lvCollections.ItemsSource = objs; 
            lvCollections.View = gridView;
        }

        public T FindVisualChild<T>(DependencyObject depObj) where T : DependencyObject
        {
            if (depObj != null)
            {
                for (int i = 0; i < VisualTreeHelper.GetChildrenCount(depObj); i++)
                {
                    DependencyObject child = VisualTreeHelper.GetChild(depObj, i);
                    if (child != null && child is T)
                    {
                        return (T)child;
                    }

                    T childItem = FindVisualChild<T>(child);
                    if (childItem != null) return childItem;
                }
            }
            return null;
        }

        private void DeleteButtonClick(object sender, RoutedEventArgs e)
        {
            var sourceButton = sender as Button;
            var dic = (Dictionary<String,Object>) sourceButton.Tag;

            var id = dic["_id"].ToString().ToOid();

            var mongo = new Mongo();
            mongo.Connect(); 
            var db = mongo.getDB(DatabaseName);
            var d = db.GetCollection("posts").FindOne(new Document() {{"_id", id}});
            mongo.Disconnect(); 

            //_dashboardService.Delete(DatabaseName,SelectedCollectionName,new Document{{"_id",id}}); 
        }
    }
}
